@@ -163,7 +163,6 @@ extern const char *flag_config;
 extern const char *flag_devices;
 extern const char *flag_directories;
 extern const char *flag_encoding;
-extern const char *flag_filter;
 extern const char *flag_format;
 extern const char *flag_format_begin;
 extern const char *flag_format_close;
@@ -185,6 +184,7 @@ extern const char *flag_stats;
 extern const char *flag_tag;
 extern const char *flag_view;
 extern std::string flag_config_file;
+extern std::string flag_filter;
 extern std::string flag_hyperlink_prefix;
 extern std::string flag_hyperlink_path;
 extern std::set<std::string> flag_config_options;
